package io.p2vman.vmodextras;

import com.google.common.base.VerifyException;
import com.mojang.blaze3d.vertex.PoseStack;
import com.mojang.logging.LogUtils;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.Font;
import net.minecraft.client.gui.GuiGraphics;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.client.event.RenderGuiOverlayEvent;
import net.minecraftforge.client.event.RenderTooltipEvent;
import net.minecraftforge.client.gui.overlay.VanillaGuiOverlay;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.eventbus.api.IEventBus;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.ModLoadingContext;
import net.minecraftforge.fml.ModLoadingException;
import net.minecraftforge.fml.ModLoadingStage;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.config.ModConfig;
import net.minecraftforge.fml.event.config.ModConfigEvent;
import net.minecraftforge.fml.event.lifecycle.FMLCommonSetupEvent;
import net.minecraftforge.fml.javafmlmod.FMLJavaModLoadingContext;
import net.minecraftforge.versions.forge.ForgeVersion;
import org.slf4j.Logger;
import io.p2vman.vmodextras.compact.Compact;
import io.p2vman.vmodextras.util.ModRegistry;

@Mod(VmodExtras.MODID)
public class VmodExtras {
    public static final String MODID = "vmodextras";

    private static final Logger LOGGER = LogUtils.getLogger();

    public static final ModRegistry REGISTRY;

    static {
        REGISTRY = new ModRegistry(MODID);

        new Compact((registry, builder) -> {

        });
    }

    private static final String MIN_FORGE_VERSION = "47.3.5";

    public VmodExtras() {
        IEventBus modEventBus = FMLJavaModLoadingContext.get().getModEventBus();

        modEventBus.addListener(this::commonSetup);
        REGISTRY.register(modEventBus);

        MinecraftForge.EVENT_BUS.register(this);
        ModLoadingContext.get().registerConfig(ModConfig.Type.COMMON, Config.COMMON_CONFIG);
    }


    private void commonSetup(final FMLCommonSetupEvent event) {
        GameRulesBlat.init();

        String currentVersion = ForgeVersion.getVersion();
        if (compareVersion(currentVersion, MIN_FORGE_VERSION) < 0) {
            throw new ModLoadingException(
                    ModLoadingContext.get().getActiveContainer().getModInfo(),
                    ModLoadingStage.ERROR,
                    "VModExras requires Forge " + MIN_FORGE_VERSION + " or higher, but found " + currentVersion,
                    new VerifyException("VModExras requires Forge " + MIN_FORGE_VERSION + " or higher, but found " + currentVersion)
            );
        }
    }


    public static int compareVersion(String v1, String v2) {
        if (v1 == null || v2 == null) {
            throw new IllegalArgumentException("Version strings must not be null");
        }

        String[] parts1 = v1.split("[.-]");
        String[] parts2 = v2.split("[.-]");

        int maxLength = Math.max(parts1.length, parts2.length);

        for (int i = 0; i < maxLength; i++) {
            String s1 = i < parts1.length ? parts1[i] : "0";
            String s2 = i < parts2.length ? parts2[i] : "0";
            try {
                int n1 = Integer.parseInt(s1);
                int n2 = Integer.parseInt(s2);
                if (n1 != n2) {
                    return Integer.compare(n1, n2);
                }
            } catch (NumberFormatException e) {
                int cmp = s1.compareTo(s2);
                if (cmp != 0) {
                    return cmp;
                }
            }
        }

        return 0;
    }

    @Mod.EventBusSubscriber(modid = MODID, bus = Mod.EventBusSubscriber.Bus.MOD)
    public static class ModConfigHandler {
        @SubscribeEvent
        public static void onConfigReload(ModConfigEvent event) {
            if (event.getConfig().getSpec() == Config.COMMON_CONFIG) {
                Config.bake();
            }
        }
    }

    @Mod.EventBusSubscriber(modid = MODID, bus = Mod.EventBusSubscriber.Bus.FORGE, value = Dist.DEDICATED_SERVER)
    public static class InteractionHandler {

        @SubscribeEvent
        public static void onRightClickItem(PlayerInteractEvent.RightClickItem event) {
            if (Config.contains(event.getItemStack())) {
                event.setCanceled(true);
            }
        }

        @SubscribeEvent
        public static void onRightClickBlock(PlayerInteractEvent.RightClickBlock event) {
            if (Config.contains(event.getItemStack())) {
                event.setCanceled(true);
            }
        }

        @SubscribeEvent
        public static void onLeftClickBlock(PlayerInteractEvent.LeftClickBlock event) {
            if (Config.contains(event.getItemStack())) {
                event.setCanceled(true);
            }
        }

    }
}
