package io.p2vman.vmodextras.cc.block.entity;

import dan200.computercraft.api.peripheral.IPeripheral;
import dan200.computercraft.shared.peripheral.modem.wireless.WirelessModemBlock;
import dan200.computercraft.shared.util.CapabilityUtil;
import dan200.computercraft.shared.util.TickScheduler;
import io.p2vman.vmodextras.cc.ComputerCraftCompact;
import io.p2vman.vmodextras.cc.net.INetwork;
import io.p2vman.vmodextras.cc.net.ModemState;
import io.p2vman.vmodextras.cc.net.Network;
import io.p2vman.vmodextras.cc.peripheral.net.NetworkModemPeripheral;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.Vec3;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.util.LazyOptional;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import static dan200.computercraft.shared.Capabilities.CAPABILITY_PERIPHERAL;

public class NetworkModemEntity extends BlockEntity {

    private static class Peripheral extends NetworkModemPeripheral
    {
        private final NetworkModemEntity entity;

        Peripheral( NetworkModemEntity entity )
        {
            super(new ModemState(() -> TickScheduler.schedule(entity.tickToken )));
            this.entity = entity;
        }

        @Nonnull
        @Override
        public Level getLevel()
        {
            return entity.getLevel();
        }

        @Nonnull
        @Override
        public Vec3 getPosition()
        {
            return Vec3.atLowerCornerOf( entity.getBlockPos().relative( entity.getDirection() ) );
        }

        @Override
        public boolean equals( IPeripheral other )
        {
            return this == other || (other instanceof NetworkModemEntity.Peripheral && entity == ((NetworkModemEntity.Peripheral) other).entity);
        }

        @Nonnull
        @Override
        public Object getTarget()
        {
            return entity;
        }

        @Override
        protected INetwork getNetwork() {
            return Network.getUniversal();
        }
    }

    private final NetworkModemPeripheral modem;
    private LazyOptional<IPeripheral> modemCap;
    @Nullable
    private Runnable modemChanged;
    private final TickScheduler.Token tickToken = new TickScheduler.Token( this );

    public NetworkModemEntity(BlockPos pos, BlockState state)
    {
        super(ComputerCraftCompact.NETWORK_MODEM.getEntity().get(), pos, state );
        this.modem = new Peripheral(this);
    }

    @Deprecated
    public void setBlockState(BlockState state) {
        Direction direction = this.getDirection();
        super.setBlockState(state);
        if (this.getDirection() != direction && this.modemChanged != null) {
            this.modemChanged.run();
        }
    }

    public void onModemChanged(Runnable callback) {
        this.modemChanged = callback;
    }

    public void setRemoved() {
        super.setRemoved();
        this.modem.removed();
    }

    public void clearRemoved() {
        super.clearRemoved();
        TickScheduler.schedule(this.tickToken);
    }

    public void blockTick()
    {
        if( modem.getModemState().pollChanged() ) updateBlockState();
    }

    @Nonnull
    private Direction getDirection()
    {
        return getBlockState().getValue(WirelessModemBlock.FACING );
    }

    private void updateBlockState()
    {
        boolean on = modem.getModemState().isOpen();
        BlockState state = getBlockState();
        if( state.getValue(WirelessModemBlock.ON ) != on )
        {
            getLevel().setBlockAndUpdate( getBlockPos(), state.setValue(WirelessModemBlock.ON, on ) );
        }
    }

    @Nonnull
    @Override
    public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side )
    {
        if( cap == CAPABILITY_PERIPHERAL )
        {
            if( side != null && getDirection() != side ) return LazyOptional.empty();
            if( modemCap == null ) modemCap = LazyOptional.of( () -> modem );
            return modemCap.cast();
        }

        return super.getCapability( cap, side );
    }
}
