package io.p2vman.vmodextras.cc.net;

import it.unimi.dsi.fastutil.objects.Object2IntArrayMap;
import it.unimi.dsi.fastutil.objects.ObjectOpenHashSet;
import lombok.Getter;

import javax.annotation.Nonnull;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

public class Network implements INetwork {
    private static Network universalNetwork = null;

    public static Network getUniversal()
    {
        if( universalNetwork == null ) universalNetwork = new Network();
        return universalNetwork;
    }

    public static void resetNetworks()
    {
        universalNetwork = null;
    }

    private final Set<IPacketReceiver> receivers = ConcurrentHashMap.newKeySet();

    @Getter
    private final Set<IDomainNameHandler> domainHandlers = new ObjectOpenHashSet<>();

    public Network() {
        domainHandlers.add(new IDomainNameHandler() {
            private final Map<String, Integer> map = new Object2IntArrayMap<>();
            private int nextId = 1024;
            @Override
            public int find(String domain) {
                if (nextId > Integer.MAX_VALUE - 1024 - Byte.MAX_VALUE*2 || domain.length()>Byte.MAX_VALUE) return -1;
                return map.computeIfAbsent(domain, k -> nextId++);
            }
        });
    }

    @Override
    public void addReceiver(@Nonnull IPacketReceiver receiver)
    {
        Objects.requireNonNull( receiver, "device cannot be null" );
        receivers.add( receiver );
    }

    @Override
    public void removeReceiver(@Nonnull IPacketReceiver receiver)
    {
        Objects.requireNonNull( receiver, "device cannot be null" );
        receivers.remove( receiver );
    }

    @Override
    public void transmit(@Nonnull NetworkPacket packet)
    {
        Objects.requireNonNull(packet, "packet cannot be null" );
        for(IPacketReceiver device : receivers )
            device.receive(packet);
    }
}
