package io.p2vman.vmodextras.cc.peripheral;

import dan200.computercraft.api.lua.*;
import edn.stratodonut.trackwork.tracks.forces.PhysicsTrackController;
import edn.stratodonut.trackwork.tracks.forces.SimpleWheelController;
import it.unimi.dsi.fastutil.ints.Int2DoubleArrayMap;
import it.unimi.dsi.fastutil.ints.Int2DoubleMap;
import it.unimi.dsi.fastutil.objects.Object2IntArrayMap;
import it.unimi.dsi.fastutil.objects.Object2IntMap;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import org.joml.Vector3d;
import org.joml.Vector3dc;
import org.squiddev.cobalt.LuaDouble;
import org.valkyrienskies.core.api.ships.ServerShip;
import org.valkyrienskies.mod.common.VSGameUtilsKt;
import io.p2vman.vmodextras.cc.block.entity.TrackWorkControllerEntity;
import io.p2vman.vmodextras.trackwork.IController;

public class TrackWorkControllerPeripheral extends PeripheralBase {
    private final TrackWorkControllerEntity entity;

    public TrackWorkControllerPeripheral(TrackWorkControllerEntity entity) {
        super("track_work_controller");
        this.entity = entity;
    }

    private ServerShip cachedShip = null;
    private Level cachedLevel = null;
    private BlockPos cachedPos = null;

    private ServerShip getCachedShip() {
        Level currentLevel = entity.getLevel();
        BlockPos currentPos = entity.getBlockPos();

        if (cachedShip == null || cachedLevel != currentLevel || !cachedPos.equals(currentPos)) {
            cachedLevel = currentLevel;
            cachedPos = currentPos;
            cachedShip = (ServerShip) VSGameUtilsKt.getShipObjectManagingPos(currentLevel, currentPos);
        }

        return cachedShip;
    }

    @LuaFunction(value = {"SetSuspensionAdjust", "setSuspensionAdjust"}, mainThread = true)
    public final void SetSuspensionAdjust(IArguments arguments) throws LuaException {
        ServerShip ship = getCachedShip();
        if (ship != null) {
            IController controller = IController.getInstance(ship, arguments.getInt(0));

            if (controller != null) {
                controller.setSuspensionAdjust(new Vector3d(
                        Math.max(Math.min(arguments.getDouble(1), 1d), 0d),
                        Math.max(Math.min(arguments.getDouble(2), 1.3d), 0.5d),
                        Math.max(Math.min(arguments.getDouble(3), 1d), 0d)
                ));
            }
        }
    }

    @LuaFunction(value = {"SetSuspensionStiffness", "setSuspensionStiffness"}, mainThread = true)
    public final void SetSuspensionStiffness(IArguments arguments) throws LuaException {
        ServerShip ship = getCachedShip();
        if (ship != null) {
            IController controller = IController.getInstance(ship, arguments.getInt(0));

            if (controller != null) {
                controller.setSuspensionStiffness((float) Math.min(Math.max(arguments.getDouble(1), 0.6d), 1.5d));
            }
        }
    }

    @LuaFunction(value = {"GetSuspensionStiffness", "getSuspensionStiffness"}, mainThread = true)
    public final MethodResult GetSuspensionStiffness(IArguments arguments) throws LuaException {
        ServerShip ship = getCachedShip();;
        if (ship != null) {
            IController controller = IController.getInstance(ship, arguments.getInt(0));

            if (controller != null) {
                return MethodResult.of(LuaDouble.valueOf(controller.getSuspensionStiffness()));
            }
        }

        return MethodResult.of();
    }

    @LuaFunction(value = {"GetSuspensionAdjust", "getSuspensionAdjust"}, mainThread = true)
    public final MethodResult GetSuspensionAdjust(IArguments arguments) throws LuaException {
        ServerShip ship = getCachedShip();
        if (ship != null) {
            IController controller = IController.getInstance(ship, arguments.getInt(0));

            if (controller != null) {
                Int2DoubleMap map = new Int2DoubleArrayMap();
                Vector3dc vec = controller.getSuspensionAdjust();

                map.put(1, vec.x());
                map.put(2, vec.y());
                map.put(3, vec.z());

                return MethodResult.of(new ObjectLuaTable(map));
            }
        }

        return MethodResult.of();
    }

    @LuaFunction(value = {"HasControllers", "hasControllers"}, mainThread = true)
    public final MethodResult HasControllers() {
        ServerShip ship = getCachedShip();
        Object2IntMap<String> map = new Object2IntArrayMap<>();
        if (ship != null) {
            if (ship.getAttachment(PhysicsTrackController.class) != null) {
                map.put("PhysicsTrack", IController.PhysicsTrack);
            }
            if (ship.getAttachment(SimpleWheelController.class) != null) {
                map.put("SimpleWheel", IController.SimpleWheel);
            }
        }
        return MethodResult.of(new ObjectLuaTable(map));
    }
}
