package io.p2vman.vmodextras.serialization;

import it.unimi.dsi.fastutil.objects.Object2ObjectArrayMap;
import lombok.experimental.Delegate;

import java.util.Map;
import java.util.Optional;

public class Codec<T> {
    @Delegate
    private final Decode<T> decode;
    @Delegate
    private final Encode<T> encode;

    public static final Map<Class<?>, Codec<?>> codecs = new Object2ObjectArrayMap<>();

    public Codec(Encode<T> encode, Decode<T> decode) {
        this.decode = decode;
        this.encode = encode;
    }

    public static final <T extends A, A> Codec<T> register(Class<T> cls, Class<A> cls2, Codec<A> codec) {
        if (!cls2.isAssignableFrom(cls)) throw new RuntimeException();
        codecs.put(cls, codec);
        return (Codec<T>) codec;
    }

    public static final <T> Codec<T> register(Class<T> cls, Codec<T> codec) {
        codecs.put(cls, codec);
        return codec;
    }

    public static <T> Optional<Codec<T>> getCodec(Class<T> cls) {
        if (!codecs.containsKey(cls)) return Optional.empty();
        try {
            return Optional.ofNullable((Codec<T>) codecs.get(cls));
        } catch (Exception e) {
            return Optional.empty();
        }
    }
}
