package io.p2vman.vmodextras.trackwork.block.entity;

import edn.stratodonut.drivebywire.wire.ShipWireNetworkManager;
import edn.stratodonut.trackwork.tracks.forces.PhysicsTrackController;
import edn.stratodonut.trackwork.tracks.forces.SimpleWheelController;

import io.p2vman.vmodextras.serialization.Context;
import io.p2vman.vmodextras.util.VsUtils;
import net.minecraft.core.BlockPos;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.Tag;
import net.minecraft.world.level.block.Rotation;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;

import org.valkyrienskies.core.api.ships.ServerShip;
import org.valkyrienskies.core.api.ships.Ship;
import org.valkyrienskies.mod.common.VSGameUtilsKt;

import io.p2vman.vmodextras.trackwork.TrackWorkCompact;


public class CompactBackupBlockEntity extends BlockEntity {
    public CompactBackupBlockEntity(BlockPos p_155229_, BlockState p_155230_) {
        super(TrackWorkCompact.COMPACT_BACKUP_BLOCK_ENTITY.get(), p_155229_, p_155230_);
    }

    @Override
    protected void saveAdditional(CompoundTag tag) {
        super.saveAdditional(tag);

        if (this.level != null) {
            Ship raw = VSGameUtilsKt.getShipObjectManagingPos(this.level, getBlockPos());
            if (raw instanceof ServerShip ship) {
                Context ctx = new Context(this, ship);
                PhysicsTrackController physicsTrackController = ship.getAttachment(PhysicsTrackController.class);
                if (physicsTrackController != null) {
                    tag.put("PhysicsTrackController", TrackBackupBlockEntity.PHYSICS_TRACK_CONTROLLER_CODEC.encode(physicsTrackController, ctx));
                }

                SimpleWheelController simpleWheelController = ship.getAttachment(SimpleWheelController.class);
                if (simpleWheelController != null) {
                    tag.put("SimpleWheelController", TrackBackupBlockEntity.SIMPLE_WHEEL_CONTROLLER_CODEC.encode(simpleWheelController, ctx));
                }
            }

            Ship raw2 = VSGameUtilsKt.getShipManagingPos(this.level, this.getBlockPos());
            if (raw2 instanceof ServerShip ship) {
                CompoundTag BackupData = new CompoundTag();

                ShipWireNetworkManager.get(ship).ifPresent((m) -> BackupData.merge(m.serialiseToNbt(this.level, this.getBlockPos())));

                tag.put("WireNetwork", BackupData);
            }
        }
    }

    @Override
    public void load(CompoundTag tag) {
        super.load(tag);

        Ship raw = VSGameUtilsKt.getShipObjectManagingPos(this.level, getBlockPos());
        if (raw instanceof ServerShip ship) {
            Context ctx = new Context(this, ship);
            if (tag.contains("SimpleWheelController", Tag.TAG_COMPOUND)) {
                VsUtils.getOrCreateAttachment(ship, SimpleWheelController.class).ifPresent((controller) -> {
                    TrackBackupBlockEntity.SIMPLE_WHEEL_CONTROLLER_CODEC.decode(tag.getCompound("SimpleWheelController"), controller, ctx);
                });
            }

            if (tag.contains("PhysicsTrackController", Tag.TAG_COMPOUND)) {
                VsUtils.getOrCreateAttachment(ship, PhysicsTrackController.class).ifPresent((controller) -> {
                    TrackBackupBlockEntity.PHYSICS_TRACK_CONTROLLER_CODEC.decode(tag.getCompound("PhysicsTrackController"), controller, ctx);
                });
            }
        }

        Ship ship = VSGameUtilsKt.getShipManagingPos(this.level, getBlockPos());
        if (ship instanceof ServerShip s) {
            if (tag.contains("WireNetwork", 10)) {
                ShipWireNetworkManager.loadIfNotExists(s, level, tag.getCompound("WireNetwork"), this.getBlockPos(), Rotation.NONE);
            }
        }
    }
}
