package io.p2vman.vmodextras.trackwork.block.entity;

import edn.stratodonut.trackwork.tracks.forces.PhysicsTrackController;
import edn.stratodonut.trackwork.tracks.forces.SimpleWheelController;
import io.p2vman.vmodextras.serialization.Codec;
import io.p2vman.vmodextras.serialization.Context;
import io.p2vman.vmodextras.trackwork.IController;
import io.p2vman.vmodextras.util.VsUtils;
import net.minecraft.core.BlockPos;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.Tag;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;
import org.joml.Vector3d;
import org.joml.Vector3dc;
import org.valkyrienskies.core.api.ships.ServerShip;
import org.valkyrienskies.core.api.ships.Ship;
import org.valkyrienskies.mod.common.VSGameUtilsKt;
import io.p2vman.vmodextras.trackwork.TrackWorkCompact;

public class TrackBackupBlockEntity extends BlockEntity {
    public TrackBackupBlockEntity(BlockPos p_155229_, BlockState p_155230_) {
        super(TrackWorkCompact.MY_BLOCK_ENTITY.get(), p_155229_, p_155230_);
    }

    public static final Codec<IController> CONTROLLER_CODEC = new Codec<>((controller, ctx) -> {

        CompoundTag root = new CompoundTag();

        root.putFloat("SuspensionStiffness", controller.getSuspensionStiffness());
        {
            CompoundTag suspensionAdjust = new CompoundTag();

            Vector3dc vec = controller.getSuspensionAdjust();

            suspensionAdjust.putDouble("x", vec.x());
            suspensionAdjust.putDouble("y", vec.y());
            suspensionAdjust.putDouble("z", vec.z());

            root.put("suspensionAdjust", suspensionAdjust);
        }

        return root;
    }, (root, controller, ctx) -> {

        if (root.contains("SuspensionStiffness", Tag.TAG_FLOAT)) {
            controller.setSuspensionStiffness(root.getFloat("SuspensionStiffness"));
        }

        if (root.contains("suspensionAdjust", Tag.TAG_COMPOUND)) {
            CompoundTag sus = root.getCompound("suspensionAdjust");
            controller.setSuspensionAdjust(new Vector3d(sus.getDouble("x"), sus.getDouble("y"), sus.getDouble("z")));
        }

        return controller;
    });

    public static final Codec<PhysicsTrackController> PHYSICS_TRACK_CONTROLLER_CODEC = Codec.register(PhysicsTrackController.class, IController.class, CONTROLLER_CODEC);

    public static final Codec<SimpleWheelController> SIMPLE_WHEEL_CONTROLLER_CODEC = Codec.register(SimpleWheelController.class, IController.class, CONTROLLER_CODEC);


    @Override
    protected void saveAdditional(CompoundTag tag) {
        super.saveAdditional(tag);
        Ship raw = VSGameUtilsKt.getShipObjectManagingPos(this.level, getBlockPos());
        if (raw instanceof ServerShip ship) {
            Context ctx = new Context(this, ship);
            PhysicsTrackController physicsTrackController = ship.getAttachment(PhysicsTrackController.class);
            if (physicsTrackController != null) {
                tag.put("PhysicsTrackController", PHYSICS_TRACK_CONTROLLER_CODEC.encode(physicsTrackController, ctx));
            }

            SimpleWheelController simpleWheelController = ship.getAttachment(SimpleWheelController.class);
            if (simpleWheelController != null) {
                tag.put("SimpleWheelController", SIMPLE_WHEEL_CONTROLLER_CODEC.encode(simpleWheelController, ctx));
            }
        }
    }

    @Override
    public void load(CompoundTag tag) {
        super.load(tag);
        Ship raw = VSGameUtilsKt.getShipObjectManagingPos(this.level, getBlockPos());
        if (raw instanceof ServerShip ship) {
            Context ctx = new Context(this, ship);
            if (tag.contains("SimpleWheelController", Tag.TAG_COMPOUND)) {
                VsUtils.getOrCreateAttachment(ship, SimpleWheelController.class).ifPresent((controller) -> {
                    SIMPLE_WHEEL_CONTROLLER_CODEC.decode(tag.getCompound("SimpleWheelController"), controller, ctx);
                });
            }

            if (tag.contains("PhysicsTrackController", Tag.TAG_COMPOUND)) {
                VsUtils.getOrCreateAttachment(ship, PhysicsTrackController.class).ifPresent((controller) -> {
                    PHYSICS_TRACK_CONTROLLER_CODEC.decode(tag.getCompound("PhysicsTrackController"), controller, ctx);
                });
            }
        }
    }
}
